<?php

namespace App\Controller;

use App\Entity\CaseItinerary;
use App\Entity\CaseLocation;
use App\Entity\Location;
use App\Entity\MemberCase;
use App\Entity\Messages;
use App\Entity\User;
use App\Entity\UserCase;
use App\Libs\Breadcrumb;
use App\Libs\Libs;
use App\Libs\NavList;
use DateTime;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Security\Http\Attribute\CurrentUser;
use Symfony\UX\Map\InfoWindow;
use Symfony\UX\Map\Map;
use Symfony\UX\Map\Marker;
use Symfony\UX\Map\Point;
use Symfony\UX\Map\Polyline;

class ItineraryController extends AbstractController
{
    private array $msgs = [];

    private int $notificationCount = 0;

    public function __construct(
        private EntityManagerInterface $entityManager,
        private array $navLinks = []
    ) {
        $this->navLinks = NavList::LIST;
    }

    #[Route('/itinerary/report', name: 'app_report_itinerary')]
    public function reportItinerary(Request $request, #[CurrentUser()] ?User $user): Response
    {
        $this->navLinks['case_itinerary'] = NavList::PRESENT_LINK;

        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($user);
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($user);

        $itineraries = $this->entityManager->getRepository(CaseItinerary::class)->getRecentTravel(
            $user
        );

        $ucs = $this->entityManager->getRepository(UserCase::class)->findBy(['user' => $user]);
        $cases = [];
        foreach ($ucs as $uc) {
            /** @var UserCase $uc */
            $cases[] = $uc->getMemberCase();
        }

        return $this->render(
            'internal/cases/itinerary/report.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_dashboard'), 'Dashboard'),
                    ],
                    'notifications' => $this->msgs,
                    'notificationCount' => $this->notificationCount,
                    'itineraries' => $itineraries,
                    'cases' => $cases
                ]
            )
        );
    }

    #[Route('/itinerary/map', name: 'app_map_itinerary')]
    public function mapItinerary(Request $request, #[CurrentUser()] ?User $user): Response
    {
        $this->navLinks['case_itinerary'] = NavList::PRESENT_LINK;

        $case = null;
        if ($request->getPayload()->get('caseId')) {
            $case = $this->entityManager->getRepository(MemberCase::class)->find($request->getPayload()->get('caseId'));
        }
        $startDate = ($request->getPayload()->get('startDate') ?new DateTime($request->getPayload()->get('startDate')) : null);
        $endDate = ($request->getPayload()->get('endDate') ? new DateTime($request->getPayload()->get('endDate')) : null);

        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($user);
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($user);

        $itineraries = $this->entityManager->getRepository(CaseItinerary::class)->getRecentTravel($user, [
            'case' => $case,
            'from' => $startDate,
            'to' => $endDate,
        ]);

        $map = new Map('default');
        $map->center(new Point($_ENV['MAP_CENTER_LAT'], $_ENV['MAP_CENTER_LON']))
            ->zoom($_ENV['MAP_ZOOM_LEVEL'])
        ;

        $total_distance = 0;
        $total_duration = new DateTime('00:00:00');

        foreach ($itineraries as $itinerary) {
            /** @var CaseItinerary $itinerary */
            $map->addMarker(new Marker(
                position: new Point(
                    $itinerary->getOriginLocation()->getLat(),
                    $itinerary->getOriginLocation()->getLon()
                ),
                title: $itinerary->getOriginLocation()->getName(),
                infoWindow: new InfoWindow(
                    content: $itinerary->getOriginLocation()->getName(),
                )
            ));

            $map->addMarker(new Marker(
                position: new Point(
                    $itinerary->getDestLocation()->getLat(),
                    $itinerary->getDestLocation()->getLon()
                ),
                title: $itinerary->getDestLocation()->getName(),
                infoWindow: new InfoWindow(
                    content: $itinerary->getDestLocation()->getName(),
                )
            ));

            /** @var CaseItinerary $itinerary */
            $map->addPolyline(new Polyline(
                points: $itinerary->getGPSPolyLines(),
                infoWindow: new InfoWindow(
                    content: $itinerary->getMemberCase()->getCaseName()
                ),
                extra: ['color' => '#FF0000']
            ));

            $total_distance += $itinerary->getDistance();
            $total_duration->add($itinerary->getDuration());
        }
        $map->fitBoundsToMarkers(true);

        $diff = new DateTime('00:00:00');
        $di = $diff->diff($total_duration);

        return $this->render(
            'internal/cases/itinerary/map.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_my_cases'), 'My Cases'),
                        new Breadcrumb($this->generateUrl('app_report_itinerary'), 'Itinerary Report'),
                    ],
                    'notifications' => $this->msgs,
                    'notificationCount' => $this->notificationCount,
                    'map' => $map,
                    'total_distance' => $total_distance,
                    'total_duration' => $di->format("%H:%i'%s''"),
                ]
            )
        );
    }

    #[Route('/api/get-case-locations/{caseId}', name: 'get_case_locations')]
    public function createItinerary(string $caseId): Response
    {
        $case = $this->entityManager->getRepository(MemberCase::class)->find($caseId);
        $cls = $this->entityManager->getRepository(CaseLocation::class)->getCaseLocations($case);

        $locations = [];
        foreach ($cls as $cl) {
            /** @var CaseLocation $cl */
            $locations[] = $cl->getLocation();
        }

        return $this->json($locations);
    }

    #[Route('/api/add-location-to-itinerary', name: 'add_location_to_itinerary')]
    public function addLocationToItinerary(Request $request, Session $session): Response
    {
        $case = $this->entityManager->getRepository(MemberCase::class)->find($request->getPayload()->get('caseId'));
        $origin = $this->entityManager->getRepository(Location::class)->find($request->getPayload()->get('origin'));
        $destination = $this->entityManager->getRepository(Location::class)->find($request->getPayload()->get('destination'));
        $departure = $request->getPayload()->get('departure');
        $arrival = $request->getPayload()->get('arrival');
        $caseMileage = (bool) $request->getPayload()->get('caseMileage');
        $date = new DateTime($request->getPayload()->get('date'));

        $route = Libs::getRouteDistance($origin, $destination);

        if (!$route) {
            return $this->json(['success' => false, 'message' => 'No route found']);
        }

        $ci = new CaseItinerary();
        $ci->setMemberCase($case)
           ->setDate($date)
           ->setCaseMileage($caseMileage)
           ->setOriginLocation($origin)
           ->setDestLocation($destination)
           ->setDeparture(new \DateTimeImmutable($departure))
           ->setArrival(new \DateTimeImmutable($arrival))
           ->setDistance($route->getDistance())
           ->setDuration($route->getDuration())
           ->setGpsRoute($route->getGeometry())
        ;

        $this->entityManager->persist($ci);
        $this->entityManager->flush();

        $session->getFlashBag()->add(
            'success',
            'Location added to itinerary'
        );
        return $this->json(['success' => true, 'message' => 'Location added to itinerary']);
    }

    #[Route('/api/filter-itinerary-by-case', name: 'ajax_filter_itinerary_by_case')]
    public function filterItineraryByCase(Request $request, #[CurrentUser()] ?User $user): Response
    {
        $case = null;
        $startDate = null;
        $endDate = null;

        //dd($request->getPayload());

        if ($request->getPayload()->get('caseId')) {
            $caseId = $request->getPayload()->get('caseId');
            $case = $this->entityManager->getRepository(MemberCase::class)->find($caseId);
        }

        if ($request->getPayload()->get('startDate')) {
            $startDate = new DateTime($request->getPayload()->get('startDate'));
        }

        if ($request->getPayload()->get('endDate')) {
            $endDate = new DateTime($request->getPayload()->get('endDate'));
        }

        $itineraries = $this->entityManager->getRepository(CaseItinerary::class)->getRecentTravel($user, [
            'case' => $case,
            'from' => $startDate,
            'to' => $endDate,
        ]);

        $ret = [];

        foreach ($itineraries as $itinerary) {
            /** @var CaseItinerary $itinerary */
            $ret[] = [
                'id' => $itinerary->getId()->toString(),
                'date' => $itinerary->getDate()->format('F j, Y'),
                'origin' => $itinerary->getOriginLocation(),
                'destination' => $itinerary->getDestLocation(),
                'distance' => $itinerary->getDistance(),
                'duration' => $itinerary->getDuration()->format("%h:%i'%s''"),
                'case' => $itinerary->getMemberCase()->getCaseName(),
            ];
        }

        return $this->json($ret);
    }
}
